from Globals import InitializeClass
import OFS
from DateTime.DateTime import DateTime
from AccessControl import ClassSecurityInfo
from NB_BaseClasses import NB_BaseItem
from NB_Utils import getDTML, getReqAttr, setFormError, setFormMsg
from cStringIO import StringIO
import os, os.path, string, types, urllib, mimetypes, sys

def copyFile(infile, outfile):
	if type(infile) is types.StringType: 
		instream = open(infile, 'rb')
		close_in = 1
	else:
		instream = infile
		close_in = 0
	if type(outfile) is types.StringType: 
		outstream = open(outfile, 'wb')
		close_out = 1
	else:
		outstream = outfile
		close_out = 0

	blocksize = 2<<16
	block = instream.read(blocksize)
	outstream.write(block)
	while len(block)==blocksize:
		block = instream.read(blocksize)
		outstream.write(block)
	try: 
		instream.seek(0)
	except: 
		pass
	
	if close_in: 
		instream.close()
	if close_out: 
		outstream.close()

def manage_addExternalFile_NB(self,
			id=None,
			title='',
			description='',
			file_path='',
			ext_file = '',
			REQUEST=None):
	"""
		Add External File
	"""

	if not id:
		return self.manage_addExternalFileForm(setFormError(REQUEST, 'id', self.getMessage_NB("Invalid ID!")))
	
	if not file_path:
		return self.manage_addExternalFileForm(setFormError(REQUEST, 'file_path', self.getMessage_NB("Invalid File Path!")))

	if not ext_file.filename:
		return self.manage_addExternalFileForm(setFormError(REQUEST, 'ext_file', self.getMessage_NB("Specify a file to be uploaded!")))

	copyFile(ext_file, file_path)
	
	instance = NB_ExternalFile(id, title, description, file_path)
	self._setObject(id, instance)
	
	if REQUEST is not None:
		return self.manage_main(self, REQUEST, update_menu=1)	
	

class NB_ExternalFile(NB_BaseItem):

	"""
	NeoBoard ExternalFile Class
	"""
	
	meta_type = 'NeoBoard External File'
	icon='misc_/NeoBoard/external_file_icon'
	
	_default_mime_type = 'application/octet-stream'
	_default_wrapper = 'File'
	
	_wrappers_list = [ 'File', 'Image'  ]
	_temp_file_ext = '_temp'
	
	manage_options = (
		{'label':'Edit', 'action': 'manage_editForm'},
		{'label':'View', 'action': 'manage_view'},
		{'label':'Security',   'action':'manage_access'},
		{'label':'Ownership',  'action':'manage_owner'},
		{'label':'Undo',       'action':'manage_UndoForm'},
	)	
	
	security = ClassSecurityInfo()
	
	security.declareObjectPublic()
        
	def __init__(self, id, title, description, file_path):
		self.id          = id       
		self.title       = title
		self.description = description
		self.file_path    = file_path

		if title:
			self.menu_title=title
		else:
			self.menu_title=id
		
		mime_type, encoding = mimetypes.guess_type(file_path)
		
		if mime_type:
			self.content_type = mime_type
		else:
			self.content_type = self._getDefaultMimeType()

        	type, subtype  = self.content_type.split('/', 2)
        
	        if type == 'image':
	            self.setWrapper('Image')
	        elif type == 'text':
	            self.setWrapper(self.getDefaultWrapper())
	        else:
	            self.setWrapper('File')
            		
	security.declareProtected('View',  'getContentType')
	def getContentType(self):
		"""
			Returns Content-type
		"""
		return self.content_type
		
	security.declareProtected('View',  'getFilePath')
	def getFilePath(self):
		"""
			Returns File Path
		"""		
		return self.file_path
	
	security.declareProtected('View',  'getFileName')
	def getFileName(self):
		"""
			Returns File Name
		"""		
		return os.path.basename(self.file_path)
	
	security.declareProtected('View',  'getFileSize')
	def getFileSize(self):
		"""
			Returns File Size
		"""		
		try:
			filesize = os.stat(self.getFilePath())[6]
		except:
			filesize = 0
		return filesize

	security.declareProtected('View',  'getFileSizeInKB')	
	def getFileSizeInKB(self):
		"""
			Returns File Size in KB
		"""		
		return round(int(self.getFileSize()/1024))
	
	security.declareProtected('View', 'getFileModTime')
	def getFileModTime(self):
		"""
			Returns File Modification Time
		"""		
		return os.path.getmtime(self.getFilePath())

	security.declareProtected('View', 'getModTime')
	def getModTime(self):
		"""
			Returns Object Modification Time
		"""				
		file_mtime = self.getFileModTime()
		if self._p_mtime > file_mtime:
			return self._p_mtime
		else:
			return file_mtime
		
	security.declareProtected('View', 'getRawContent')
	def getRawContent(self):
		"""
			Returns Raw Content
		"""				
		data = StringIO()
		
		copyFile(self.getFilePath(), data)

		return data.getvalue()
	
	security.declareProtected('Edit External File', 'setRawContent')
	def setRawContent(self, content):
		"""
			Sets Raw Content
		"""				
		data = StringIO(content)
		copyFile(data, self.getFilePath())
	
	security.declareProtected('Access contents information', 'index_html')
	def index_html(self, REQUEST=None):
		"""
			Return index_html
		"""
		
		return self.__call__(None, REQUEST)
	
	security.declareProtected('Access contents information', '__str__')
	def __str__(self, REQUEST=None):
		return self.__call__(None, REQUEST)
			
	def __len__(self): 
		return 1		
	
	security.declareProtected('Access contents information', '__call__')
	def __call__(self, client=None, REQUEST=None):
		"""
			Returns index_html
		"""	

		wrapper = self.getWrapper()
		data = self.getRawContent()
		
		if wrapper == 'Image':
			resultImage = OFS.Image.Image(self.id, 
						self.title,
			                        data,
			                        self.getContentType())
			resultImage._p_mtime     = self.getModTime()
			resultImage.absolute_url = self.absolute_url
			resultImage.content_type = self.getContentType()

			if REQUEST is not None:			
				resp = REQUEST['RESPONSE'] 
				resp.setHeader('Content-Disposition',' inline; filename=' + self.getTitle() + '-' + self.getFileName())				
				return resultImage.index_html(REQUEST, resp)
			else:
				return resultImage.__str__()

		elif wrapper == 'File':
			resultFile = OFS.Image.File(self.id, 
						self.title,
			                        data,
			                        self.getContentType())

			resultFile._p_mtime     = self.getModTime()
			resultFile.absolute_url = self.absolute_url
			resultFile.content_type = self.getContentType()
			if REQUEST is not None:
				resp = REQUEST['RESPONSE']
				resp.setHeader('Content-Disposition',' inline; filename=' + self.getTitle() + '-' + self.getFileName())
				return resultFile.index_html(REQUEST, resp)
			else:
				return resultFile.__str__()
	
	security.declarePublic('__len__')
	def __len__(self): 
		return 1
		
	security.declareProtected('Edit External File', 'manage_editExternalFile')
	def manage_editExternalFile(self, 
			title = '',
			description = '',
			wrapper = '',
			ext_file = '',
			file_path = '',
			REQUEST=None):
		"""
			Edit External File
		"""
		
		if title:
			self.title = title
		if description:
			self.description = description

		if title:
			self.menu_title=title
		else:
			self.menu_title=id
		
		if wrapper:
			self.setWrapper(wrapper)
			
		if file_path and (not file_path == self.getFilePath()):
			copyFile(self.getFilePath(), file_path)
			try:
				self._delete_file()
			except:
				pass
					
			self.file_path = file_path

			mime_type, encoding = mimetypes.guess_type(file_path)
		
			if mime_type:
				self.content_type = mime_type
			else:
				self.content_type = self._getDefaultMimeType()

	        	type, subtype  = self.content_type.split('/', 2)
	        
		        if type == 'image':
		            self.setWrapper('Image')
		        elif type == 'text':
		            self.setWrapper(self.getDefaultWrapper())
		        else:
		            self.setWrapper('File')
			
		if ext_file.filename:
			self.setRawContent(ext_file.read())
			
		if REQUEST is not None:
			return self.manage_editForm(self, REQUEST)


	security.declareProtected('Edit External File', 'manage_deleteExternalFile')
	def manage_deleteExternalFile(self, REQUEST=None):
		"""
			Deletes External File
		"""
				
		self.aq_inner.aq_parent._delObject(self.id)
		
		if REQUEST is not None:
			return self.manage_editExternalFile(title, description, wrapper, content, REQUEST)
	
	security.declarePrivate('_delete_file')
	def _delete_file(self):
		try:
			os.unlink(self.getFilePath())
		except:
			pass		
		self._delThumbnail()
		
	security.declarePrivate('_make_temp_file')
	def _make_temp_file(self):
		try:
			os.rename(self.getFilePath(), self.getFilePath() + '.' + self._getTempFileExt())
		except:
			pass		
		self._delThumbnail()
		
	def manage_beforeDelete(self, item, container):
		"""
			Puts the file in a temporary state
		"""		
		
		self._make_temp_file()
		
		return NB_ExternalFile.inheritedAttribute ("manage_beforeDelete")(self, item, container)
		
	def manage_afterClone(self, item):
		"""
			Copy and pasted files get renamed with a time stamp
		"""	
	
		try: 
			self.absolute_url(1)
		except: 
			self._v_has_been_cloned=1
		else:
			if os.path.isfile(self.getFilePath()) and not hasattr(self, '_v_copy_exists'):
				new_file_path = self._getNewFilePath()
				copyFile(self.getFilePath(), new_file_path)
				self.file_path = new_file_path

		return NB_ExternalFile.inheritedAttribute ("manage_afterClone") (self, item)
				
	def manage_afterAdd(self, item, container):
		"""
			Checks if the object is being copied
		"""				
		self._restore()
		if hasattr(self, "_v_has_been_cloned"):
			delattr(self, "_v_has_been_cloned")
			self.manage_afterClone(item)
		return NB_ExternalFile.inheritedAttribute ("manage_afterAdd")(self, item, container)

	security.declarePrivate('_restore')
	def _restore(self):
		""" 
			Restores filename after undo or copy-paste 
		"""
		
		if not os.path.isfile(self.getFilePath()) and os.path.isfile(self.getFilePath()+ '.' + self._getTempFileExt()): 
			os.rename(self.getFilePath()+'.' + self._getTempFileExt(), self.getFilePath())

	security.declarePrivate('_getNewFilePath')
	def _getNewFilePath(self):
		"""
			Returns a new file path with a timestamp
		"""				
		new_file_path =  os.path.splitext(self.getFilePath())[0] + '_' + DateTime().strftime('%Y%m%d%H%M%S') + '_.' + os.path.splitext(self.getFilePath())[1]
		while 1:
			if os.path.isfile(new_file_path):
				new_file_path =  os.path.splitext(self.getFilePath())[0] + '_' + DateTime().strftime('%Y%m%d%H%M%S') + '.' + os.path.splitext(self.getFilePath())[1]
			else:
				break
		
		return new_file_path
	
	security.declarePublic('getWrappersList')
	def getWrappersList(self):
		"""
			Returns the list of wrappers
		"""		
		return self._wrappers_list

	security.declarePublic('setWrapper')
	def setWrapper(self, wrapper):
		"""
			Sets the wrapper
		"""		
		self._wrapper = wrapper
	
	security.declarePublic('getWrapper')
	def getWrapper(self):
		"""
			Returns the wrapper
		"""		
		if self._wrapper in self.getWrappersList():
			return self._wrapper
		else:
			self.setWrapper(self.getDefaultWrapper())
			return self._wrapper

	security.declarePublic('getDefaultWrapper')
	def getDefaultWrapper(self):
		"""
			Returns the default wrapper
		"""		
		return self._default_wrapper

	security.declarePublic('thumb')
	def thumb(self, REQUEST=None):
		"""
			Returns thumbnail of this file
		"""
		
		if not self.getWrapper() == 'Image':
			return ''
	
		if REQUEST is not None:
			max_width = getReqAttr(REQUEST, 'max_width')		
			max_height = getReqAttr(REQUEST, 'max_height')		
			keep_ratio = getReqAttr(REQUEST, 'keep_ratio')		
			force = getReqAttr(REQUEST, 'force')
		
		if not max_width:
			max_width = 10
		if not max_height:
			max_height = 10
		
		if keep_ratio is None:
			keep_ratio = 1
		
		if force is None:
			force = 0
		
		try:
			thumbnail_file = self._createThumbnail(image_file=self.getFilePath(), max_width=int(max_width), max_height=int(max_height), keep_ratio=int(keep_ratio), force=int(force))		
			resultThumbnail = OFS.Image.Image(self.id, 
						self.title,
			                        open(thumbnail_file, 'rb').read(),
			                        self.getContentType())
			resultThumbnail._p_mtime     = self.getModTime()
			resultThumbnail.absolute_url = self.absolute_url
			resultThumbnail.content_type = 'image/jpeg'
	
			if REQUEST is not None:			
				resp = REQUEST['RESPONSE'] 
				return resultThumbnail.index_html(REQUEST, resp)
			else:
				return resultThumbnail.__str__()		
		except:
			return ''

	security.declarePrivate('_getTempFileExt')
	def _getTempFileExt(self):
		return self._temp_file_ext


	security.declarePrivate('_getDefaultMimeType')
	def _getDefaultMimeType(self):
		"""
			Returns the default mime type
		"""		
		return self._default_mime_type

	def getImageSize(self, image_file=None):
		"""
			Returns the size of the image
		"""		
		if image_file is None:	
			if not self.getWrapper() == 'Image':
				return 0,0
			if hasattr(self, 'image_width') and hasattr(self, 'image_height'):
				if self.image_width > 0 and self.image_height > 0:
					return self.image_width, self.image_height
			
			try:
				from PIL import Image
	
				im = Image.open(self.getFilePath()) 
				self.image_width = im.size[0]
				self.image_height = im.size[1]
			except:
				self.image_width = self.image_height = 0
			return self.image_width, self.image_height
		
		else:
			try:
				im = Image.open(image_file) 
				image_width = im.size[0]
				image_height = im.size[1]
			except:
				image_width = image_height = 0
			
			return image_width, image_height
			
	
	def _createThumbnail(self, image_file, max_width, max_height, keep_ratio=0, force=0):
		"""
			Generates a thumbnail of the image
		"""	
		save_file_name = os.path.splitext(image_file)[0] + '_thumb' + '.jpg'
		width, height = self.getImageSize()
		if width <= max_width and height <= max_height:
			return image_file
		
		if os.path.isfile(save_file_name) and not force:
			width, height = self.getImageSize(save_file_name)
			if width == max_width and height == max_height:
				return save_file_name
	
		from PIL import Image
		im = Image.open(image_file) 
		if im.mode!='RGB' and im.mode!='CMYK': 
			im = im.convert("RGB")
		
		if keep_ratio==1:
			im.thumbnail((max_width,max_height))
		else: 					
			im.draft(None,(max_width,max_height))
			im.load()
			im = im.resize((max_width,max_height), Image.BICUBIC)
		
		im.save(save_file_name,'JPEG', quality = 100)
		
		return save_file_name

	def _delThumbnail(self):
		"""
			Deletes the thumbnail of the file
		"""		
		thumnail_file = os.path.splitext(self.getFilePath())[0] + '_thumb' + '.jpg'
		try:
			os.unlink(thumnail_file)
		except:
			pass

	security.declareProtected('selfUpgrade', 'Edit NeoBoard')	
	def selfUpgrade(self):
		pass
	
InitializeClass(NB_ExternalFile)
