# NeoBoard Theme

from Globals import package_home, InitializeClass
from AccessControl import ClassSecurityInfo	
from Globals import MessageDialog
from OFS.Folder import Folder
from OFS.DTMLMethod import DTMLMethod
from NB_Utils import getDTML, addObject, addImage, setFormError, setFormMsg
from NB_BaseClasses import NB_BaseItem
from os import listdir
import re, os, types

StringType=type('')
ListType=type([])

manage_addNeoBoardThemeFolderForm_NB = getDTML('dtml/ThemeFolderAdd')
manage_editNeoBoardThemeFolderForm_NB = getDTML('dtml/ThemeFolderEdit')

def manage_addNeoBoardThemeFolder_NB(self, 
			id='', 
			title='',
			REQUEST=None):
	""" Add NeoBoard Theme Folder """

	if not id:
		return self.manage_addNeoBoardThemeFolderForm_NB(setFormError(REQUEST, 'id', "Invalid ID!"))				
	
	if not title:
		title = id
		
	theme_folder = NB_ThemeFolder(title)
	theme_folder.id = id
        self._setObject(id, theme_folder)
        
        theme_folder = self._getOb(id)
        theme_folder._initThemes()
        
	if REQUEST is not None:
		return self.manage_main(self, REQUEST, update_menu=1)
				

class NB_ThemeFolder(Folder, NB_BaseItem):

	meta_type='NeoBoard Theme Folder'
    	icon='misc_/NeoBoard/theme_folder_icon'

	meta_types=(
		{'name': 'NeoBoard Theme', 'action': 'manage_addNeoBoardThemeForm_NB'},		
	)

	all_meta_types = meta_types
	
	manage_options=(
		{'label':'Contents',       'action':'manage_main'},	
		{'label':'Properties', 'action':'manage_editNeoBoardThemeFolderForm_NB'},
		{'label':'Security',   'action':'manage_access'},
		{'label':'Ownership',  'action':'manage_owner'},
		{'label':'Undo',  'action':'manage_undoForm'},
	)	
	
	
	def __init__(self, title=None):
		self.title=title

    	def _get_id(self, id):
    		"""
    			Conjure up a temporary filename for an object being cut and pasted
    		"""
    		
    		return id + '.TEMP'
	
	def getThemes_NB(self):
		return self.objectValues('NeoBoard Theme')		
	
	def manage_editNeoBoardThemeFolder(self, 
			title='',
			reset_default_themes=0,
			REQUEST=None):
		""" Edit NeoBoard Theme Folder """

		if not title:
			title = id
			
		self.title = title
		
		if reset_default_themes:	
	        	self._initThemes()
	        
		if REQUEST is not None:
			return self.manage_main(self, REQUEST, update_menu=1)
	
	def _initThemes(self):
		theme_path = package_home(globals())+ '/themes/'

		for fname in listdir(theme_path):
			if os.path.isdir(theme_path + fname) and os.path.isfile(theme_path + fname + '/theme_name.txt'):
				theme = NB_Theme()
				theme.id = fname
				theme.enabled = 1
				theme.addThemeFromFile(theme_path + fname + '/')
				if hasattr(self, theme.id):
					self.manage_delObjects([theme.id,])
					
				self._setObject(fname, theme)
				self._getOb(fname).initMessages()	

class NB_ThemeMessageFile(DTMLMethod):

	meta_type = 'NeoBoard Theme Message File'
	id = 'messages'
	
	def __init__(self, file=None):
		if file:
			self.munge(file)
		else:
			self.munge('#Empty Theme Message File')		
	
	def manage_edit(self,data,title,SUBMIT='Change',dtpref_cols='50',
	            dtpref_rows='20',REQUEST=None):
		"""
			Calls initMessages when saved
		"""

		ret_value = NB_ThemeMessageFile.inheritedAttribute("manage_edit")(self,data,title,SUBMIT,dtpref_cols,dtpref_rows,REQUEST)

		if hasattr(self, 'initMessages'):
			self.initMessages()
		
		if ret_value:
			return ret_value
				
	def manage_upload(self,file='', REQUEST=None):
		"""
			Calls initMessages when saved
		"""
		ret_value = NB_ThemeMessageFile.inheritedAttribute("manage_upload")(self,file, REQUEST)
		
		if hasattr(self, 'initMessages'):
			self.initMessages()

		if ret_value:
			return ret_value			

manage_addNeoBoardThemeForm_NB=getDTML('dtml/ThemeAdd')

def manage_addNeoBoardTheme_NB(self, 
		inherit_from='default',
		id='',
		title='', 
		enabled=0,
		authors=[],
		backup_themes=[],
		REQUEST=None):
	""" Add NeoBoard Theme"""
	
	if not id:
		return self.manage_addThemeForm(setFormError(REQUEST, 'id', self.getMessage_NB("Invalid ID!")))
	
	pointer = self.manage_copyObjects([inherit_from,])
	self.manage_pasteObjects(pointer)
	self.manage_renameObject(inherit_from + '.TEMP', id)

	theme = getattr(self, id)
	
	theme.title = title
	
	if title:
		theme.menu_title = title
	else:
		theme.menu_title = id
	

	result = theme.setBackupThemes(backup_themes)
	if result is not None:
		return self.manage_addThemeForm(setFormError(REQUEST, 'backup_themes', self.getMessage_NB(result)))
	
	theme.setAuthors(authors)
	theme.enabled = enabled

	if REQUEST is not None:
		return self.manage_main(self, REQUEST, update_menu=1)
		
class NB_Theme(Folder, NB_BaseItem):

	meta_type='NeoBoard Theme'
    	icon='misc_/NeoBoard/theme_icon'

	manage_options=(
		{'label':'Contents',   'action':'manage_main'},
		{'label':'Properties', 'action':'manage_editForm'},
		{'label':'Security',   'action':'manage_access'},
		{'label':'Ownership',  'action':'manage_owner'},
		{'label':'Undo',  'action':'manage_UndoForm'},
	)

	meta_types=(
		{'name': 'NeoBoard Theme Message File', 'action': 'manage_addNeoBoardThemeMessageFile'},
		{'name': 'DTML Document', 'action': 'manage_addProduct/OFSP/documentAdd'},
		{'name': 'DTML Method', 'action': 'manage_addProduct/OFSP/methodAdd'},
		{'name': 'File', 'action': 'manage_addProduct/OFSP/fileAdd'},
		{'name': 'Folder', 'action': 'manage_addProduct/OFSP/folderAdd'},
		{'name': 'Image', 'action': 'manage_addProduct/OFSP/imageAdd'},	
		{'name': 'Page Template', 'action': 'manage_addProduct/PageTemplates/manage_addPageTemplateForm'},			
	)

	all_meta_types = meta_types
	_msg_map = {}
	
	security = ClassSecurityInfo()	
	security = ClassSecurityInfo()
	
	security.declareObjectProtected('View')
	security.declareProtected('View', 'index_html')
	
	security.declareProtected('Edit NeoBoard Theme', 'manage_editForm')
	manage_editForm=getDTML('dtml/ThemeEdit')	

	def __init__(self):
		pass
		
	def manage_addNeoBoardThemeMessageFile(self, file=None, REQUEST=None):
		"""
			Add a Theme Message File
		"""
		if hasattr(self.getSelf(), 'messages'):
			return MessageDialog(title='File Already Exists!',
		                             message='Theme Message File Already Exists!',
		                             action=REQUEST['URL1']+'/manage_main'
		                            )
			
		instance = NB_ThemeMessageFile(file)
		instance.title = 'NeoBoard Theme Message File'
		self._setObject('messages', instance)
		if REQUEST is not None:
			return self.manage_main(self, REQUEST, update_menu=1)
		
	def addThemeFromFile(self, dir):
		"""
			Used when the whole theme folder gets created
		"""
		
		if os.path.isfile(dir + 'theme_name.txt'):
			lines = open(dir + 'theme_name.txt').readlines()
			if len(lines) > 0 and lines[0].strip():
				self.title = lines[0].strip()
			else:
				self.title = self.id

		for fname in listdir(dir):
			if fname == ('messages.dtml'):
				f = open(dir + fname)
				self.manage_addNeoBoardThemeMessageFile(f.read())
				f.close() 				
			else:
				id = title = fname.replace('.dtml','').replace('.zpt', '').replace('.gif', '').replace('.jpg','').replace('.png','')
				
				addObject(self, id, title, dir + fname)

		if os.path.isdir(dir + 'icons'):
	        	self.manage_addFolder('icons','Theme Icons')
	        
	       		icon_folder = self._getOb('icons')
	
			for fname in listdir(dir + 'icons'):
				id = title = fname.replace('.gif', '').replace('.jpg','').replace('.png','')
				addObject(icon_folder, id, title, dir + 'icons/' + fname)

		if os.path.isdir(dir + 'images'):
	        	self.manage_addFolder('images','Theme Images')
	        
	       		images_folder = self._getOb('images')
	
			for fname in listdir(dir + 'images'):
				id = title = fname.replace('.gif', '').replace('.jpg','').replace('.png','')
				addObject(images_folder, id, title, dir + 'images/' + fname)

		if os.path.isdir(dir + 'templates'):
	        	self.manage_addFolder('templates','Theme Templates')		
	        	
	       		template_folder = self._getOb('templates')
	
			for fname in listdir(dir + 'templates'):
				id = title = fname.replace('.zpt', '').replace('.dtml','')
				addObject(template_folder, id, title, dir + 'templates/' + fname)
	
		self.authors=[]
		self.backup_themes=[]

	def manage_editTheme(self,
		        title='', 
			enabled=0,
			authors=[],
			backup_themes=[],
			REQUEST=None):
			
		""" Edit NeoBoard Theme"""
	
		self.title = title
		
		if title:
			self.menu_title = title
		else:
			self.menu_title = self.id
		
		result = self.setBackupThemes(backup_themes) 
		if result is not None:
			return self.manage_editForm(setFormError(REQUEST, 'backup_themes', self.getMessage_NB(result)))	
		
		self.setAuthors(authors)
		self.enabled=enabled

		if REQUEST is not None:
			return self.manage_editForm(setFormMsg(REQUEST, 'Changes applied!'))		

	def setBackupThemes(self, themes):
		"""
			Backup Themes are used when the current theme doesn't have a requested object
		"""
		
		if type(themes) == ListType:
			for theme in themes:
				if not theme in self.getThemeIDs():
					return self.getMessage_NB("Theme <em>%s</em> not found.") % theme
			self.backup_themes = themes
		else:
			themes_list = themes.split('\n')
			for theme in themes_list:
				if not theme in self.getThemeIDs():
					return self.getMessage_NB("Theme <em>%s</em> not found.") % theme
			self.backup_themes = themes_list
		
		return None
			
	def getBackupThemes(self):
		"""
			Returns the list of backup themes
		"""
		
		return self.backup_themes

	def setAuthors(self, authors):
		"""
			Sets the list of theme authors
		"""
		
		if type(authors) == ListType:
			self.authors = authors
		else:
			self.authors = authors.split('\n')

	def getAuthors(self):
		"""
			Returns the list of theme authors
		"""
		
		return self.authors

	def initMessages(self):
		"""
			Initializes messages in this theme
		"""
		
		self._msg_map = {}
		if self.id == 'default': 
			if hasattr(self, 'messages'):
				self.parseMessageFile(self._getOb('messages'))
				return
			else:
				raise 'Parse Error', "default theme doesn't have a message DTML file!"
		else:
			if hasattr(self, 'aq_parent') and hasattr(self.aq_parent, 'default') and hasattr(self.aq_parent.default, 'messages'):
				self.parseMessageFile(self.aq_parent.default._getOb('messages'))
			else:
				raise 'Parse Error', "default theme doesn't have a message DTML file!"
			
			if hasattr(self, 'messages'):
				self.parseMessageFile(self._getOb('messages'))
			else:
				raise 'Parse Error', "theme <em>%s</em> doesn't have a message DTML file!" % self.title_or_id()
		
		return self.getMessageMap()

	def parseMessageFile(self, msg_obj):  		
		"""
			Parses the message file into a map
		"""
		    		
    		key_re = re.compile(r'^ *_KEY_ *"(.*?[^\\"]*)"', re.IGNORECASE | re.MULTILINE)
    		trans_re = re.compile(r'^ *_TRANS_ *"(.*?[^\\"]*)"', re.IGNORECASE | re.MULTILINE)
    		comment_re = re.compile(r'^ *[#;]', re.IGNORECASE | re.MULTILINE)
    		blank_re = re.compile(r'^$', re.IGNORECASE | re.MULTILINE)
    		
    		try:
       			text = msg_obj.read()
       		except:
			raise 'Parse Error', "object <em>%s</em> doesn't seem to be a message DTML file! Its meta type is <em>%s</em>" % (msg_obj.title_or_id(), msg_obj.meta_type)
		
        	match_obj = key_re.search(text, 0)
        	if not match_obj:
			return
		
		msg_key = ''
		
	        while match_obj:
            				
    			msg_key = match_obj.group(1)
    			
    			start =  match_obj.end(0)
    			text = text[start:]
    			
    			match_obj = trans_re.search(text, 0)
    			
			if not match_obj:
				raise 'Parse Error', "trans not found for key <em>%s</em>" % (msg_key,)
				
    			trans = match_obj.group(1)

    			start =  match_obj.end(0)
    			text = text[start:]
    			
    			self.setMessage(msg_key, trans)
			match_obj = key_re.search(text, 0)
		
	def getMessageMap(self):
		"""
			Returns the current message map
		"""
			
		return self._msg_map
		
	def setMessage(self, key, trans):
		"""
			Sets a single key/translation pair
		"""
			
		self._msg_map[key] = trans

	def _getMessage(self, key):
		"""
			Returns the translated message for the requested key
		"""
			
		#try:
		return self._msg_map[key]
		#except:
		#	return key

	def _getThemeObject(self, id, obj_type=None, css_class=None, render=1):
		"""
			Returns the requested object 
		"""
			
		if obj_type == 'icons':
			return self.getIcon(id=id, css_class=css_class, render=render)

		elif obj_type == 'images':
			return self.getImage(id=id, css_class=css_class, render=render)
			
		elif obj_type == 'templates':
			return self.getTemplate(id=id)
		else:			
			if hasattr(self.getSelf(), id):
				return getattr(self, id)
		return None
		
	def getIcon(self, id, css_class='', render=1):
		"""
			Returns the requested icon
		"""
		
		if hasattr(self.aq_base, 'icons'):
			if hasattr(self.aq_base.icons, id):
				if render:
					return getattr(self.icons, id).tag(css_class=css_class)
				else:
					return getattr(self.icons, id)
		else:
			return None

	def getImage(self, id, css_class='', render=1):
		"""
			Returns the requested image
		"""
		
		if hasattr(self.aq_base, 'images'):
			if hasattr(self.aq_base.images, id):
				if render:
					return getattr(self.images, id).tag(css_class=css_class)
				else:
					return getattr(self.images, id)
		else:
			return None

	def getTemplate(self, id):
		"""
			Returns the requested template
		"""
			
		if hasattr(self.aq_base, 'templates'):
			if hasattr(self.aq_base.templates, id):
				return getattr(self.templates, id)
		else:
			return None


	def getFillerImage(self, vspace=1, hspace=1):
		"""
			Returns the filler image (transparent 1x1 pixel) 
		"""
			
		return self.getImage(id='filler_image', hspace=hspace, vspace=vspace)
		
	def selfUpgrade(self):
		pass		
		
InitializeClass(NB_Theme)
