# NeoBoard Article Class
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager, Role
from DateTime.DateTime import DateTime
from Products.ZSQLMethods.SQL import SQL
from NB_Utils import getDTML, getNextElementName, stripTags, traverseFolder, setFormError, getReqAttr, setFormMsg, cutString, isEmailValid
from NB_BaseClasses import NB_BaseContentFolder
from NB_ExternalFile import copyFile
import os, mimetypes

def addNeoBoardArticle_NB(self, 
		username=None,
		password=None,
		author_email=None,
		title='', 
		data='',
		category='',
		attachments=[],
		mail_reply=0,
		is_notice=0,
		REQUEST=None):
	""" Add NeoBoard Article"""

	if self.isReadOnly():
		if not self.isNeoBoardManager():
			return self.post(REQUEST=setFormError(REQUEST, 'title',  self.getMessage_NB("It's a read only board.")))
	
	if not self.useACLUserAuth():
		if not username:
			return self.post(setFormError(REQUEST, 'username', self.getMessage_NB('The field cannot be empty!')))	
		
		if not self.emptyPasswordAllowed() and not password:
			return self.post(setFormError(REQUEST, 'password', self.getMessage_NB('The field cannot be empty!')))	

	if not self.emptyEmailAllowed():
		if not author_email:
			return self.post(setFormError(REQUEST, 'author_email', self.getMessage_NB('The field cannot be empty!')))	

		if not isEmailValid(author_email):
			return self.post(setFormError(REQUEST, 'author_email', self.getMessage_NB('Invalid Email Address!')))
			
	if not title:
		return self.post(setFormError(REQUEST, 'title', self.getMessage_NB('The field cannot be empty!')))

	if not self.emptyBodyAllowed() and not data:
		return self.post(setFormError(REQUEST, 'data', self.getMessage_NB('The field cannot be empty!')))	
	
	if is_notice:
		instance = NeoBoardNotice()
		id = 'n_' + str(self.getNextArticleNum(1))

	else:
		instance = NeoBoardArticle()
		id = 'a_' + str(self.getNextArticleNum())

	instance.id = id	
	instance.title=title
	
	instance.menu_title=instance.getTitle()
			
	instance.munge(data)
	instance.mail_reply=mail_reply
	instance.setCategory(category)
	instance.setReadCount(0)
	instance.setNextReplyNum(0)
	instance.setTotalNumReplies(0)
	instance.setTotalNumVotes(0)

	if self.autoApproveArticle() or is_notice:
		instance.approved = 1
	else:
		instance.approved = 0
		
	if self.useACLUserAuth():
		instance.author=REQUEST.AUTHENTICATED_USER.getUserName()
	else:
		instance.author=username
		if password:
			instance.password=password
		else:
			instance.password=str(DateTime())


	instance.author_email=author_email
			
	if self.attachmentsAllowed() and attachments:
		f_list = []
		for attachment in attachments:
			f_list.append(attachment.file)

		if not self.checkSize(f_list):
			return self.post(REQUEST=setFormError(REQUEST, 'attachment', self.getMessage_NB('File too big!')))	
		
	self._setObject(id, instance)
	
	if self.attachmentsAllowed() and attachments:
		for attachment in attachments:
			if attachment.file.filename:
				if os.path.splitext(attachment.file.filename)[1] in self.getBadExtensions():
					self.manage_delObjects([id,])
					return self.post(REQUEST=setFormError(REQUEST, 'attachment', self.getMessage_NB('The following extensions are not allowed: %s.') % ' '.join(self.getBadExtensions())))	

				self._getOb(id).uploadFile(attachment.file, attachment.title, attachment.description)
	
	
	self.setTotalNumArticles(self.getTotalNumArticles()+1)
	
	if self.isMailEnabled():
		self._getOb(id).mailArticle()

	self._getOb(id)._setRemoteIP(REQUEST)

	if REQUEST is not None:
    		return REQUEST.RESPONSE.redirect(self.absolute_url() + self._getOb(id)._getPageQueryString(REQUEST))		

class NeoBoardArticle(NB_BaseContentFolder):
	"""
	NeoBoard Article Class
	"""
	
	meta_type='NeoBoard Article'
	icon='misc_/NeoBoard/neoboard_article_icon'     
	
	_max_thread_depth = 5
	
	manage_options=({'label':'Contents', 'icon':'', 'action':'manage_main', 'target':'manage_main'},
			{'label':'View', 'icon':'', 'action':'index_html', 'target':'manage_main'},                 
			{'label':'Properties', 'action':'manage_editForm'},
			{'label':'Security', 'icon':'', 'action':'manage_access', 'target':'manage_main'},
			{'label':'Undo', 'icon':'', 'action':'manage_UndoForm', 'target':'manage_main'},
	)  

	security = ClassSecurityInfo()
	security.declareObjectProtected('View')
	
	def __init__(self):
		pass

	def reply(self, REQUEST=None):
		"""
			Returns Reply Template
		"""
		if REQUEST is not None:
			return apply(self.getThemeObject_NB(id='NeoBoardArticleReplyAdd', obj_type='templates', render=0), (self, REQUEST))
		else:
			return apply(self.getThemeObject_NB(id='NeoBoardArticleReplyAdd', obj_type='templates', render=0), (self, self.REQUEST))

	def modify(self, REQUEST=None):
		"""
			Returns Modify Template
		"""		
		if REQUEST is not None:
			return apply(self.getThemeObject_NB(id='NeoBoardArticleModify', obj_type='templates', render=0), (self, REQUEST))
		else:
			return apply(self.getThemeObject_NB(id='NeoBoardArticleModify', obj_type='templates', render=0), (self, self.REQUEST))		

	def delete_form(self, REQUEST=None):
		"""
			Returns Delete Template
		"""		
		if REQUEST is not None:
			return apply(self.getThemeObject_NB(id='NeoBoardArticleDelete', obj_type='templates', render=0), (self, REQUEST))
		else:
			return apply(self.getThemeObject_NB(id='NeoBoardArticleDelete', obj_type='templates', render=0), (self, self.REQUEST))		

	def manage_editForm(self, REQUEST=None):
		"""
			Returns Edit Template
		"""				
		if REQUEST is not None:
			return apply(self.getThemeObject_NB(id='NeoBoardArticleEdit', obj_type='templates', render=0), (self, REQUEST))
		else:
			return apply(self.getThemeObject_NB(id='NeoBoardArticleEdit', obj_type='templates', render=0), (self, self.REQUEST))		
			
	def index_html(self, REQUEST={}, RESPONSE=None):
		"""
			Returns index_html
		"""
		
		if not self.isEnabled() and not self.isNeoBoardManager():
			return self.getMessage_NB('The Board has Expired.')
		
		if self.isNeoBoardManager() and self._upgradeNeeded():
			return self._upgradeForm()
		
		index_template = apply(self.getThemeObject_NB(id='NeoBoardArticle', obj_type='templates', render=0), (self, self.REQUEST)) 
		
		REQUEST.set('CURRENT_ARTICLE_URL', self.absolute_url())
		REQUEST.set('ARTICLE_IS_APPROVED', self.isApproved())

		header = apply(self.getThemeObject_NB(id='NeoBoardHeader', obj_type='templates', render=0), (self, self.REQUEST))   			
	        footer = apply(self.getThemeObject_NB(id='NeoBoardFooter', obj_type='templates', render=0), (self, self.REQUEST))   			
		self.readCountUp()
		
      		return header + index_template + footer

	def __call__(self, client=None, REQUEST={}, RESPONSE=None, **kw):      
		"""
			Returns index_html
		"""		
					
		return apply(self.getThemeObject_NB(id='NeoBoardArticle', obj_type='templates', render=0), (self, self.REQUEST)) 
		
	def vote(self, REQUEST, RESPONSE):
		"""
			Votes for this article
		"""		
		
		self.setTotalNumVotes(self.getTotalNumVotes()+1)
		return self.index_html(REQUEST, RESPONSE)
		
	def getReadCount(self):
		"""
			Returns read count
		"""		
		
		return self.read_count

	def setReadCount(self, count=0):
		"""
			Sets read count to the given number
		"""		
		
		self.read_count = count
	
	def readCountUp(self):
		"""
			Increases read count by one
		"""		
		
		self.setReadCount(self.getReadCount() + 1)
		
	def _p_resolveConflict(self, oldState, savedState, newState):
		"""
			Attempts to resolve possible read/write conflicts
		"""		
		
	        saved_diff = savedState['read_count'] - oldState['read_count']
	        new_diff = newState['read_count']- oldState['read_count']
	
	        oldState['read_count'] = oldState['read_count'] + saved_diff + new_diff
	
	        return oldState		
		
	def getAuthor(self):
		"""
			Returns author of this article
		"""		
		
		return self.author

	def getAuthorEmail(self):
		"""
			Returns author email when ACL user authentication is not in use
		"""		
		
		if hasattr(self, 'author_email'):
			return self.author_email
		else:
			return ''

	def getPassword(self):
		"""
			Returns article password when ACL user authentication is not in use
		"""				
		if hasattr(self, 'password'):
			return self.password
		else:
			return ''

	def getNeoBoardArticleURL(self):
		"""
			Returns relative URL of this article
		"""				
		url = self.absolute_url()
		return url.replace(self.getNeoBoardURL() + '/', '')			

	def getDepth(self):
		"""
			Returns how deep the article goes in the current thread
		"""			
			
		depth = self.getNeoBoardArticleURL().count('/')
		if depth > self._max_thread_depth:
			return self._max_thread_depth
		else:
			return depth
	
	def getParent(self):
		"""
			Returns parent of this article
		"""				
		if self.isThreadParent():
			return None
			
		return NeoBoardArticle.inheritedAttribute("getParent")(self)

	def getThreadParent(self):
		"""
			Returns top-most parent of this article
		"""						
		if self.isThreadParent():
			return self

		parents = self.REQUEST.PARENTS
		for i in xrange(len(parents)-1):
			parent = parents[i]
        		if parent.isThreadParent():
        			return parent
		return None
		
	def getArticleNum(self):
		"""
			Returns article number
		"""						
		if self.isThreadParent():
			return self.id.replace('a_', '')
		else:
			return '-'
		
	def getNumAttachments(self):
		"""
			Returns the number of attachments
		"""		
		return len(self.getAttachments())
		
	def getAttachmentTypeIcon(self, id=1):
		"""
			Returns attachment mime type icon
		"""	
		if not self.getNumAttachments() > 0:
			return '-'

		try:		
			mime_type, encoding = mimetypes.guess_type(self._getOb('FILE_' + str(id)).getFileName())
		except:
			return '-'

		if not mime_type:
			return self.getThemeObject_NB(id='mime_unknown', obj_type='icons')
		
        	type, subtype  = mime_type.split('/', 2)
        
	        if type == 'image':
			return self.getThemeObject_NB(id='mime_img', obj_type='icons')
	        elif type == 'text':
	        	if subtype == 'html':
	            		return self.getThemeObject_NB(id='mime_html', obj_type='icons')
	        	elif subtype == 'richtext':
	        		return self.getThemeObject_NB(id='mime_docs', obj_type='icons')
	            	else:
	            		return self.getThemeObject_NB(id='mime_text', obj_type='icons')
	        elif type == 'audio':
	        	return self.getThemeObject_NB(id='mime_audio', obj_type='icons')
	        elif type == 'video':
	        	return self.getThemeObject_NB(id='mime_video', obj_type='icons')
	        elif type == 'application':
	        	if subtype == 'zip' or subtype == 'x-tar' or subtype == 'x-gtar' or subtype == 'x-gzip':
	        		return self.getThemeObject_NB(id='mime_zip', obj_type='icons')
	        	elif subtype == 'msword' or subtype == 'rtf':
	        		return self.getThemeObject_NB(id='mime_docs', obj_type='icons')
	        	else:
	        		return self.getThemeObject_NB(id='mime_bin', obj_type='icons')
	        else:
			return self.getThemeObject_NB(id='mime_unknown', obj_type='icons')
	
	def hasChildren(self):
		"""
			Tells if the article has replies
		"""						
		if len(self.objectValues('NeoBoard Article')) > 0:
			return 1
		else:
			return 0
	
	def isThreadParent(self):
		"""
			Tells if the article is top-most one in the current thread
		"""						
		if self.aq_parent.meta_type == 'NeoBoard':
			return 1
		else:
			return 0

	def isOwner(self):
		"""
			Tells if the user is owner of this article
		"""						
		if self.REQUEST.AUTHENTICATED_USER.has_role('Manager', self):
			return 1
			
		if self.useACLUserAuth() and self.REQUEST.AUTHENTICATED_USER.has_role('Owner', self):
			return 1

		return 0
	
	security.declarePrivate('_getPageQueryString')			
	def _getPageQueryString(self, REQUEST):
		"""
			Cooks query string to be returned in REQUEST object
		"""					
		if REQUEST is None:
			return ''
			
		queries = []

		if REQUEST.has_key('expand_all'):
			queries.append('expand_all=' + REQUEST['expand_all'])		
		
		if REQUEST.has_key('start'):
			queries.append('start=' + REQUEST['start'])

		if REQUEST.has_key('skey'):
			queries.append('skey=' + REQUEST['skey'])			

		if REQUEST.has_key('reverse'):
			queries.append('reverse=' + REQUEST['reverse'])

		query_string = ''
				
		if len(queries) > 0:
			query_string = '&'.join(queries)
			
			query_string = '?' + query_string
		
		return query_string
		
		
	security.declareProtected('View NeoBoard Article', 'manage_deleteNeoBoardArticle')
	def manage_deleteNeoBoardArticle(self, 
			password=None,
			REQUEST=None):
		""" Delete NP NeoBoard Article"""		                            

		if not self.useACLUserAuth() and not self.isNeoBoardManager():
			if not password:
				return self.delete_form(setFormError(REQUEST, 'password', self.getMessage_NB('The field cannot be empty!')))

			if not password == self.getPassword():
				return self.delete_form(setFormError(REQUEST, 'password', self.getMessage_NB('Password Mismatch!')))		

		if self.hasChildren():
			return MessageDialog(title=self.getMessage_NB('Undeletable'),
	                             message=self.getMessage_NB("You can't delete an article with replies."),
	                             action=self.absolute_url() + self._getPageQueryString(REQUEST)
	                )


		if not self.isOwner() and not self.isNeoBoardManager():
			return MessageDialog(title=self.getMessage_NB('Undeletable'),
	                             message=self.getMessage_NB("You don't have permissions to delete this article."),
	                             action=self.absolute_url() + self._getPageQueryString(REQUEST)
	                )
	
		ret_url = self.aq_inner.aq_parent.absolute_url() + self._getPageQueryString(REQUEST)
		
		self.aq_inner.aq_parent._delObject(self.id)
		self.getCurrentNeoBoard()._resetNeoBoard()

		if REQUEST is not None:
            		return REQUEST.RESPONSE.redirect(ret_url)

	security.declareProtected('View NeoBoard Article', 'delete')
	delete = manage_deleteNeoBoardArticle

	security.declareProtected('View NeoBoard Article', 'manage_modifyNeoBoardArticle')            		
	def manage_modifyNeoBoardArticle(self, 
			username=None,
			password=None,
			author_email=None,
			title='', 
			data='',
			category='',
			attachments=[],
			mail_reply=0,
			REQUEST=None):
		""" Modify NP NeoBoard Article"""
		
		if self.useACLUserAuth():
			if not self.isOwner():
				return self.index_html(REQUEST=setFormError(REQUEST, 'title', self.getMessage_NB("You don't have permissions to modify this article.")))
		else:			
			if not username:
				return self.modify(setFormError(REQUEST, 'username', self.getMessage_NB('The field cannot be empty!')))
			
			if not password and not self.isNeoBoardManager():
				return self.modify(setFormError(REQUEST, 'password', self.getMessage_NB('The field cannot be empty!')))	

			if not password == self.getPassword() and not self.isNeoBoardManager():
				return self.modify(setFormError(REQUEST, 'password', self.getMessage_NB('Password Mismatch!')))	
					
		if not self.emptyEmailAllowed():
			if not author_email:
				return self.modify(setFormError(REQUEST, 'author_email', self.getMessage_NB('The field cannot be empty!')))	

			if not isEmailValid(author_email):
				return self.modify(setFormError(REQUEST, 'author_email', self.getMessage_NB('Invalid Email Address!')))

		if not self.isOwner() and not self.isApproved():
			return self.modify(setFormError(REQUEST, 'title', self.getMessage_NB('The article has yet to be approved!')))

		if not title:
			return self.modify(setFormError(REQUEST, 'title', self.getMessage_NB('The field cannot be empty!')))
	
		if not self.emptyBodyAllowed() and not data:
			return self.modify(setFormError(REQUEST, 'data', self.getMessage_NB('The field cannot be empty!')))
				
		self.title=title
		self.menu_title=self.getTitle()
		self.munge(data)
		self.mail_reply=mail_reply
		
		if not self.useACLUserAuth():
			self.author = username
		
		self.author_email = author_email
		
		if self.isThreadParent():
			self.setCategory(category)
		else:
			self.category=''
		
		if self.attachmentsAllowed() and attachments:
			f_list = []
			idx=1
			extra=0
			
			for attachment in self.getAttachments():
				extra = extra + attachment.getFileSize()
				
			for attachment in attachments:
				t_file = self.getAttachment(idx)
				if t_file is not None:
					extra = extra - t_file.getFileSize()
				
				if attachment.file.filename:
					f_list.append(attachment.file)
				idx = idx + 1
					
	
			if not self.checkSize(f_list, extra=extra):
				return self.post(REQUEST=setFormError(REQUEST, 'attachment', self.getMessage_NB('File too big!')))	
		
			idx=1
			for attachment in attachments:

				if attachment.file.filename:
					if os.path.splitext(attachment.file.filename)[1] in self.getBadExtensions():
						return self.modify(REQUEST=setFormError(REQUEST, 'attachment', self.getMessage_NB('The following extensions are not allowed: %s.') % ' '.join(self.getBadExtensions())))
			
				t_file = self.getAttachment(idx)
				if t_file is not None:
					self.uploadFile(attachment.file, attachment.title, attachment.description, id=idx)
				else:			
					if attachment.file.filename:	
						self.uploadFile(attachment.file, attachment.title, attachment.description)
				idx=idx + 1
		
		self.updateContent()
		self.reindex_object()
		
		if not self.isNeoBoardManager():
			self._setRemoteIP(REQUEST)
		
		if REQUEST is not None:
            		return REQUEST.RESPONSE.redirect(self.absolute_url() + self._getPageQueryString(REQUEST))		

	security.declareProtected('Edit NeoBoard', 'manage_editNeoBoardArticle')
	def manage_editNeoBoardArticle(self, 
			username=None,
			password=None,
			author_email=None,
			title='', 
			data='',
			category='',
			read_count=0,
			author=None,
			creation_date=None,
			mail_reply=0,
			total_num_votes=0,
			attachments=[],
			approved=0,
			REQUEST=None):
		""" Edit NP NeoBoard Article"""

		if self.useACLUserAuth():
			if not author:
				return self.manage_editForm(REQUEST=setFormError(REQUEST, 'author', self.getMessage_NB("The field cannot be empty!")))
		else:			
			if not username:
				return self.manage_editForm(setFormError(REQUEST, 'username', self.getMessage_NB('The field cannot be empty!')))
			
			if not password:
				return self.manage_editForm(setFormError(REQUEST, 'password', self.getMessage_NB('The field cannot be empty!')))	
					
		if not self.emptyEmailAllowed():
			if not author_email:
				return self.manage_editForm(setFormError(REQUEST, 'author_email', self.getMessage_NB('The field cannot be empty!')))	

			if not isEmailValid(author_email):
				return self.manage_editForm(setFormError(REQUEST, 'author_email', self.getMessage_NB('Invalid Email Address!')))

		self.author_email = author_email
			
		if not title:
			return self.manage_editForm(setFormError(REQUEST, 'title', self.getMessage_NB('The field cannot be empty!')))
	
		if not self.emptyBodyAllowed() and not data:
			return self.manage_editForm(setFormError(REQUEST, 'data', self.getMessage_NB('The field cannot be empty!')))

				
		self.title=title
		self.menu_title=self.getTitle()
		self.munge(data)
		self.mail_reply=mail_reply
		
		if not self.useACLUserAuth():
			self.author = username

			self.password=password
		else:
			if not self.acl_users.userExists(author.strip()):
				return self.manage_editForm(REQUEST=setFormError(REQUEST, 'author', 'No such user!'))
			else:
				self.author=author.strip()
		
		if self.isThreadParent():
			self.setCategory(category)
		else:
			self.category=''

		self.read_count=read_count		
		self.setTotalNumVotes(total_num_votes)
		self.approved=approved
		
		if self.attachmentsAllowed() and attachments:
			f_list = []
			idx=1
			extra=0
			
			for attachment in self.getAttachments():
				extra = extra + attachment.getFileSize()
				
			for attachment in attachments:
				t_file = self.getAttachment(idx)
				if t_file is not None:
					extra = extra - t_file.getFileSize()
				
				if attachment.file.filename:
					f_list.append(attachment.file)
				idx = idx + 1
					
	
			if not self.checkSize(f_list, extra=extra):
				return self.post(REQUEST=setFormError(REQUEST, 'attachment', self.getMessage_NB('File too big!')))	
		
			idx=1
			for attachment in attachments:
				t_file = self.getAttachment(idx)
				if t_file is not None:
					self.uploadFile(attachment.file, attachment.title, attachment.description, id=idx)
				else:			
					if attachment.file.filename:	
						self.uploadFile(attachment.file, attachment.title, attachment.description)
				idx=idx + 1			

		self.updateContent()
		self.reindex_object()
		
		if REQUEST is not None:
			return self.manage_editForm(REQUEST=setFormMsg(REQUEST, 'Changes applied!'))

	security.declareProtected('Edit NeoBoard', 'manage_approve')
	def manage_approve(self, REQUEST=None):
		"""
			Disapproves the article
		"""		
		self.approved = 1
		if REQUEST is not None:
			return self.index_html(REQUEST=setFormMsg(REQUEST, self.getMessage_NB('Article Approved!')))

	security.declareProtected('Edit NeoBoard', 'manage_disapprove')
	def manage_disapprove(self, REQUEST=None):
		"""
			Approves the article
		"""
		
		self.approved = 0
		if REQUEST is not None:
			return self.index_html(REQUEST=setFormMsg(REQUEST, self.getMessage_NB('Article Disapproved!')))


	security.declareProtected('View NeoBoard Article', 'manage_addNeoBoardArticleReply')	
	def manage_addNeoBoardArticleReply(self, 
			username=None,
			password=None,
			author_email=None,	
			title='', 
			data='',
			attachments='',
			mail_reply=0,			
			REQUEST=None):
		""" Add NP NeoBoard Article Reply"""		         

		if self.isReadOnly():
			if not self.isNeoBoardManager():
				return self.reply(REQUEST=setFormError(REQUEST, 'title',  self.getMessage_NB("It's a read only board.")))
		
		if not self.useACLUserAuth():
			if not username:
				return self.reply(setFormError(REQUEST, 'username', self.getMessage_NB('The field cannot be empty!')))	
			
			if not self.emptyPasswordAllowed() and not password:
				return self.reply(setFormError(REQUEST, 'password', self.getMessage_NB('The field cannot be empty!')))	
	
		if not self.emptyEmailAllowed():
			if not author_email:
				return self.reply(setFormError(REQUEST, 'author_email', self.getMessage_NB('The field cannot be empty!')))	

			if not isEmailValid(author_email):
				return self.reply(setFormError(REQUEST, 'author_email', self.getMessage_NB('Invalid Email Address!')))

						

		if not self.isApproved():
			return self.reply(setFormError(REQUEST, 'title', self.getMessage_NB("You can't reply to an unapproved article.")))
				
		if not title:
			return self.reply(setFormError(REQUEST, 'title', self.getMessage_NB('The field cannot be empty!')))
	
		if not self.emptyBodyAllowed() and not data:
			return self.reply(setFormError(REQUEST, 'data', self.getMessage_NB('The field cannot be empty!')))	
		
		instance = NeoBoardArticle()
		id = 'a_' + str(self.getNextReplyNum())				
	
		instance.id = id	
		instance.title=title
		
		instance.menu_title=instance.getTitle()
				
		instance.munge(data)
		instance.mail_reply=mail_reply
		instance.setReadCount(0)
		instance.setNextReplyNum(0)
		instance.setTotalNumReplies(0)
		instance.setTotalNumVotes(0)		
		
		if self.autoApproveArticle():
			instance.approved = 1
		else:
			instance.approved = 0		
	
		if self.useACLUserAuth():
			instance.author=REQUEST.AUTHENTICATED_USER.getUserName()
		else:
			instance.author=username
			if password:
				instance.password=password
			else:
				instance.password=str(DateTime())

		instance.author_email=author_email
					
		instance.creation_date = DateTime()
	
		if self.attachmentsAllowed() and attachments:
			f_list = []
			for attachment in attachments:
				f_list.append(attachment.file)
	
			if not self.checkSize(f_list):
				return self.post(REQUEST=setFormError(REQUEST, 'attachment', self.getMessage_NB('File too big!')))	
			
		self._setObject(id, instance)
		
		if self.attachmentsAllowed() and attachments:
			idx = 1
			for attachment in attachments:
				if attachment.file.filename:
					if os.path.splitext(attachment.file.filename)[1] in self.getBadExtensions():
						self.manage_delObjects([id,])
						return self.post(REQUEST=setFormError(REQUEST, 'attachment', self.getMessage_NB('The following extensions are not allowed: %s.') % ' '.join(self.getBadExtensions())))

					self._getOb(id).uploadFile(attachment.file, attachment.title, attachment.description)
									
		self.setNextReplyNum(self.getNextReplyNum())
		self.setTotalNumReplies(self.getTotalNumReplies()+1)		
		
		
		if self.isMailEnabled():
			self._getOb(id).mailArticle()
			
		if self.mail_reply:
			self.mailArticle(reply=1, reply_instance=self._getOb(id))
		
		self._getOb(id)._setRemoteIP(REQUEST)		
		
		if REQUEST is not None:
            		return REQUEST.RESPONSE.redirect(self.absolute_url() + self._getPageQueryString(REQUEST))

	def manage_afterAdd(self, item, container):
		"""
			Makes sure attachments are properly copied or moved
		"""
	
		self.creation_date = DateTime()
		if self.hasAttachment():
			for ext_file in self.getAttachments():
				org_ext_file_path = ext_file.getFilePath()
				new_ext_file_path = self.getArchivePath() + self.cookFilePath(ext_file.getId()) +  os.path.splitext(org_ext_file_path)[1]
				
				# Must have been copied or moved
				if not os.path.isfile(new_ext_file_path):
					if os.path.isfile(org_ext_file_path + '.' + ext_file._getTempFileExt()):
						# Moved
						copyFile(org_ext_file_path + '.' + ext_file._getTempFileExt(), new_ext_file_path)
						os.unlink(org_ext_file_path + '.' + ext_file._getTempFileExt())
					else:
						# Copied
						# NB_External file attempts to create a time-tagged copy of itself when cloned.
						# It ensures deep-copying of the object. 
						# In this case, however, the path of the actual file on the filesystem differs so that the above-mentioned mechanism is unnecessary.
						# _v_copy_exists volatile property prevents this from happening
						copyFile(org_ext_file_path, new_ext_file_path)						
						ext_file._v_copy_exists=1

					ext_file.file_path = new_ext_file_path

		self.updateContent()
		self.reindex_object()
										
		NeoBoardArticle.inheritedAttribute("manage_afterAdd")(self, item, container)

	def getSlicedTitle(self):
		"""
			Returns title reduced to Maximum List Title Length
		"""						
		depth = self.getDepth()
		
		chunk = self.getMaxListTitleLength() - (self.getDepth() * 2)
		
		return cutString(self.getTitle(), chunk)	
								
	def getNextReplyNum(self):
		"""
			Returns next reply article number
		"""						
		return self.next_reply_num + 1

	def setNextReplyNum(self, num):
		"""
			Sets next reply article number
		"""						
		self.next_reply_num = num
	
	def setTotalNumReplies(self, num):
		"""
			Sets the total number of replies of this article
		"""						
		self.total_num_replies = num 

	def getTotalNumReplies(self):
		"""
			Returns the total number of replies of this article
		"""						
		return self.total_num_replies
		
	def setTotalNumVotes(self, num):
		"""
			Sets the total number of votes for this article
		"""						
		self.total_num_votes = num 

	def getTotalNumVotes(self):
		"""
			Returns the total number of votes for this article
		"""								
		return self.total_num_votes
		
	def mailArticle(self, reply=0, reply_instance=None):
		"""
			Mails this article
		"""
											
		if reply and reply_instance is not None:

			subject = self.getMessage_NB('Your Article Replied!') + ' - ' + reply_instance.getTitle()
			
			body = apply(self.getThemeObject_NB(id='NeoBoardArticleReplyMail', obj_type='templates', render=0), (reply_instance, self.REQUEST))
			
			author_email = self.getAuthorEmail()
			reply_email = reply_instance.getAuthorEmail()
			
			if isEmailValid(author_email):
				if isEmailValid(reply_email):
					self.sendmail(from_addr='[NeoBoard Mailer]', to_addr=author_email, reply_addr=reply_email, subject=subject, body=body, html=1)
				else:
					self.sendmail(from_addr='[NeoBoard Mailer]', to_addr=author_email, reply_addr='', subject=subject, body=body, html=1)		
		else:
			if self.getCurrentNeoBoard().getManagers() > 0:

				subject = self.getMessage_NB('NeoBoard Article') + ' - ' + self.getTitle()
				
				body = apply(self.getThemeObject_NB(id='NeoBoardArticleMail', obj_type='templates', render=0), (self, self.REQUEST))
				
				managers = self.getCurrentNeoBoard().getManagers()
				if len(managers) > 0:
					for manager in 	managers:
						manager_email = manager.getEmail()
						if isEmailValid(manager_email):
							self.sendmail(from_addr='[NeoBoard Mailer]', to_addr=manager_email, reply_addr=portal_name, subject=subject, body=body, html=1)
					

	def getNextFileID(self):
		"""
			Returns next attachment file ID 
		"""								
		
		
		l = self.objectValues('NeoBoard External File')
		
		next_id = len(l) + 1
		
		for id in xrange(1, next_id):
			if not hasattr(self, 'FILE_' + str(id)):
				return 'FILE_' + str(id)
		
		return 'FILE_' + str(next_id)
		
		
			
	def cookFilePath(self, file_name):
		"""
			Cooks file path for attachment filename
		"""								
		name = self.getNeoBoardArticleURL().replace('/', '_')

		return name + '__' + file_name
	
	def uploadFile(self, file, title, description, id=None):	
		"""
			Adds attachment to this article instantiating an external file object
		"""								
		desc_file_name = os.path.basename(file.filename)
				
		if not title:
			title = desc_file_name
			
		if not description:
			description = desc_file_name	

		archive_file_name = os.path.basename(file.filename.replace(' ', '_'))				

		if id:
			old_id = 'FILE_' + str(id)
			if not file.filename and not title and not description:
				self._getOb(old_id).manage_deleteExternalFile()
			else:
				
				self._getOb(old_id).manage_editExternalFile(title=title, description=description, ext_file=file)

		else:
			next_id = self.getNextFileID()
			archive_file_path = self.getArchivePath() + self.cookFilePath(next_id) +  os.path.splitext(archive_file_name)[1]				
			self.manage_addExternalFile_NB(id=next_id, title=title, description=description, file_path=archive_file_path, ext_file=file)
	
	def hasAttachment(self):
		"""
			Tells if the article has an attachment
		"""								
		if self.getNumAttachments() > 0:
			return 1
		else:
			return 0

	def getAttachments(self):
		"""
			Returns all external files
		"""								
		return self.objectValues('NeoBoard External File')

			
	def getAttachment(self, id=1):
		"""
			Returns attached file
		"""								
		try:
			file = self._getOb('FILE_' + str(id))
		except:
			file = None
		
		return file
		
	def getAttachmentURL(self, id=1):
		"""
			Returns the URL of the namedc attached file
		"""								
		if self.hasAttachment():
			if hasattr(self, 'FILE_' + str(id)):
				return self.absolute_url() + '/' + 'FILE_' + str(id)
			else:
				return ''
		else:
			return ''


	def getAttachmentTitle(self, id=1):
		"""
			Returns the title of the attached file
		"""										
		if not self.hasAttachment():
			return ''
		try:
			file = self._getOb('FILE_' + str(id))
			return file.getTitle()
		except:	
			return ''

	def getAttachmentDescription(self, id=1):
		"""
			Returns the description of the attached file
		"""										
		
		if not self.hasAttachment():
			return ''		
		try:
			file = self._getOb('FILE_' + str(id))
			return file.getDescription()
		except:	
			return ''

			
	def setCategory(self, category):
		"""
			Sets the category of this article
		"""										
		
		if category:
			self.category = category

	def getCategory(self):
		"""
			Returns the category of this article
		"""										
		
		if hasattr(self, 'category'):
			if self.category not in self.getPostingCategories().keys():
				self.setCategory('')
		
			if self.category:
				return self.category
			else:
				return ''
		else:
			return ''	

	def getCategoryName(self):
		"""
			Returns the name of the article's category
		"""										
		
		cat = self.getCategory()
		
		if cat:
			try:
				return self.posting_categories[cat]
			except:
				return ''
		else:
			return ''
			
	def getArticleSummary(self, size=250, ellipsis='...'):
		"""
			Returns article summary
		"""
		
		try:
			summary = stripTags(self.getRenderedContent()).replace('"', ' ')
			
			if len(summary) < size:
				ret_size = len(summary)
			else:
			    	ret_size = summary.find("\n\n", 0, size)
				if ret_size == -1:
					ret_size = summary.rfind(" ", 0, size)
					if ret_size == -1:
						ret_size = size
			return summary[:ret_size] + ellipsis
		except:
			return self.getTitle()
			
	def _setRemoteIP(self, REQUEST=None):
		"""
			Sets Remote IP of the poster
		"""										

		if REQUEST.has_key('REMOTE_HOST') and REQUEST['REMOTE_HOST']:
			self.remote_ip = REQUEST['REMOTE_HOST']
		elif self.REQUEST.has_key('REMOTE_ADDR') and REQUEST['REMOTE_ADDR']:
			self.remote_ip = REQUEST['REMOTE_ADDR']
		else:
			self.remote_ip = None

	def getRemoteIP(self):
		"""
			Returns the IP address of the poster
		"""										
		
		if self.remote_ip:
			return self.remote_ip
		else:
			return 'Unknown'
	
	def isApproved(self):
		"""
			Tells if the article is approved
		"""		
		if hasattr(self, 'approved'):
			return self.approved
		else:
			return 0
			
	def approveThread(self):
		"""
			Approves all the articles down this thread
		"""				
		self.approved = 1
		for article in traverseFolder(self, 'NeoBoard Article'):
			article.approved = 1

	def disApproveThread(self):
		"""
			Disapproves all the articles down this thread
		"""				
		self.approved = 0
		for article in traverseFolder(self, 'NeoBoard Article'):
			article.approved = 0		
		
	def _upgradeNeeded(self):
		if not hasattr(self.getCurrentNeoBoard(), '_upgraded_up_to'):
			return 1
			
		if not self.getCurrentNeoBoard().__NB_VERSION__ == self.getCurrentNeoBoard()._upgraded_up_to:
			return 1
		else:
			return 0

	def _upgradeForm(self):

		form_str = "<html><body><div align=\"center\">"
		form_str = form_str + "<ul><li>Please update the board to date.</li></ul>"
		form_str = form_str + "<form action=\"" + self.getCurrentNeoBoard().absolute_url()+  "/selfUpgrade\" method=\"post\">\n"
		form_str = form_str + "<input type=\"submit\" name=\"submit\" value=\"Upgrade to version " + self.getCurrentNeoBoard().__NB_VERSION__ + "\">"
		form_str = form_str + "</form>\n"
		form_str = form_str + "</div></body></html>\n"
		
		return form_str		
		
	def selfUpgrade(self):		
				
		if not hasattr(self, 'remote_ip'):
			self.remote_ip = 'Unknown'
			
		if hasattr(self.aq_base, 'attachment'):
			del self.attachment

		if hasattr(self.aq_base, 'attachment_title'):
			del self.attachment_title

		if hasattr(self.aq_base, 'attachment_description'):
			del self.attachment_description					

		if hasattr(self.aq_base, 'attachment_file_id'):
			del self.attachment_file_id
			
		self.approved = 1
		

class NeoBoardNotice(NeoBoardArticle):

	meta_type='NeoBoard Notice'	
	
InitializeClass(NeoBoardArticle)
InitializeClass(NeoBoardNotice)

